# -*- coding: utf-8 -*-
"""
Visualization functions for MCS_052_main.py.

@author: Silvan Schwebke
"""

from numpy import *
from mayavi.mlab import plot3d, points3d, mesh, figure, clf, view, show, gcf

from traits.api import HasTraits, Range, Instance, \
        on_trait_change, Bool, Button, Int
from traitsui.api import View, Item, Group, HSplit

from mayavi.core.api import PipelineBase
from mayavi.core.ui.api import MayaviScene, SceneEditor, \
                MlabSceneModel

def build_path_xyz(clusters, path, diameters_fixed, ax):
    path_x = 0.0
    path_y = clusters[path[1],1]
    path_z = clusters[path[1],2]
    for i in range(1,size(path)-1):
        path_x = hstack((path_x,clusters[path[i],0]))
        path_y = hstack((path_y,clusters[path[i],1]))
        path_z = hstack((path_z,clusters[path[i],2]))
    path_x = hstack((path_x,ax))
    path_y = hstack((path_y,clusters[path[-2],1]))
    path_z = hstack((path_z,clusters[path[-2],2]))
                   
    path_diameters = hstack((0.,diameters_fixed[path[1:-1]],0.))
          
    # Pfadanfang/-ende auf doppelte Koordinaten überprüfen (führt zu Fehlermeldung)
    if path_x[0]==path_x[1]:
        path_x = path_x[1:]
        path_y = path_y[1:]
        path_z = path_z[1:]
        path_diameters = path_diameters[1:]
    if path_x[-1]==path_x[-2]:
        path_x = path_x[:-1]
        path_y = path_y[:-1]
        path_z = path_z[:-1]
        path_diameters = path_diameters[:-1]  

    return path_x, path_y, path_z, path_diameters

def MCS_call_model(deformation,ax,ay,az,coordx,coordy,coordz,diameters_fixed,clusters0,clustersL,clustersT):

    class MyModel(HasTraits):
        axM = ax
        button_state = Int
        set_unstrained = Button()
        set_strain_longitudinal = Button()
        set_strain_transversal = Button()
        path_state = Bool
        toggle_path = Button()
        
        def _set_unstrained_fired(self):
            self.button_state = 0
        def _set_strain_longitudinal_fired(self):
            self.button_state = 1
        def _set_strain_transversal_fired(self):
            self.button_state = 2
        def _toggle_path_fired(self):
            self.path_state = not self.path_state
    
        scene = Instance(MlabSceneModel, (), {'background' : (1,1,1)})
        
        #fig=figure(figure=self.scene.scene,bgcolor=(1,1,1))
    
        plot1 = Instance(PipelineBase)
        plot2 = Instance(PipelineBase)
        plot3 = Instance(PipelineBase)
        plot4 = Instance(PipelineBase)
        plot5 = Instance(PipelineBase)
        
        sphere_resolution = 13  # Auflösung der 3D-Kugeln: 6 für gute Performance, 12 für runde Darstellung
        sphere_opacity = 1.0
        
        # When the scene is activated, or when the parameters are changed, we
        # update the plot.
        @on_trait_change('button_state,scene.activated,path_state')
        def update_plot(self):
            if self.plot1 is None:
                #self.scene.mlab.figure(1, bgcolor=(1, 1, 1), fgcolor=(0, 0, 0), size=(1200, 800))
                #clf()
                self.path_state = True
                clf()
                if az>0.0:
#                    self.plot1 = self.scene.mlab.plot3d((0,ax,ax,0,0),(0,0,ay,ay,0),(0,0,0,0,0),
#                       color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
#                    self.plot2 = self.scene.mlab.plot3d((0,ax,ax,0,0),(0,0,ay,ay,0),(az,az,az,az,az),
#                           color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
#                    self.plot3 = self.scene.mlab.plot3d((0,0),(0,0),(0,az),
#                           color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
#                    self.plot4 = self.scene.mlab.plot3d((ax,ax),(0,0),(0,az),
#                           color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
#                    self.plot5 = self.scene.mlab.plot3d((ax,ax),(ay,ay),(0,az),
#                           color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
#                    self.plot6 = self.scene.mlab.plot3d((0,0),(ay,ay),(0,az),
#                           color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
                             
                    self.plot1 = self.scene.mlab.points3d(coordx, coordy, coordz, diameters_fixed, color=(0.2, 0.2, 0.2),
                             scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=self.sphere_opacity)
                else:
                    # stattdessen Säulen-Darstellung raussuchen?
                    self.plot1 = self.scene.mlab.points3d(coordx, coordy, coordz, diameters_fixed, color=(0.2, 0.2, 0.2),
                             scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=self.sphere_opacity)
                 
#                path_x, path_y, path_z, path_diameters = build_path_xyz(clusters0, path0, diameters_fixed, ax)
#                self.plot2 = self.scene.mlab.points3d(path_x,path_y,path_z, path_diameters, color=(1.0, 0.2, 0.2),
#                     scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=1.0)    
#                self.plot3 = self.scene.mlab.plot3d(path_x,path_y,path_z,ones(len(path_x)),color=(0.2,0.9,0.2),tube_radius=0.4, opacity=1.0)
                
                # Elektrodenflächen separat als Rechtecke darstellen
                if az>0.0:
                    x1 = [[0., 0.],[0., 0.]]
                    x2 = [[ax, ax],[ax, ax]]
                    y = [[0., 0.],[ay, ay]]
                    z = [[0., az],[0., az]]
                    self.plot4 = self.scene.mlab.mesh(x1, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
                    self.plot5 = self.scene.mlab.mesh(x2, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
                else:
                    self.plot4 = self.scene.mlab.plot3d((0,0),(0,ay),(0,0), color=(0.1,0.1,0.9),tube_radius=0.5, opacity=0.5)
                    self.plot5 = self.scene.mlab.plot3d((ax,ax),(0,ay),(0,0), color=(0.1,0.1,0.9),tube_radius=0.5, opacity=0.5)
                gcf().scene.parallel_projection = True
            else:
                if self.button_state == 0:
#                    path_x, path_y, path_z, path_diameters = build_path_xyz(clusters0, path0, diameters_fixed, ax)
                    x1 = [[0., 0.],[0., 0.]]
                    x2 = [[ax, ax],[ax, ax]]
                    y = [[0., 0.],[ay, ay]]
                    z = [[0., az],[0., az]]
                    cameradata = view()
                    clf()
                    self.plot1 = self.scene.mlab.points3d(coordx, coordy, coordz, diameters_fixed, color=(0.2, 0.2, 0.2),
                         scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=self.sphere_opacity)
#                    if self.path_state:
#                        self.plot2 = self.scene.mlab.points3d(path_x,path_y,path_z, path_diameters, color=(1.0, 0.2, 0.2),
#                             scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=1.0)    
#                        self.plot3 = self.scene.mlab.plot3d(path_x,path_y,path_z,ones(len(path_x)),color=(0.2,0.9,0.2),tube_radius=0.4, opacity=1.0)
                    if az>0.0:
                        self.plot4 = self.scene.mlab.mesh(x1, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
                        self.plot5 = self.scene.mlab.mesh(x2, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
                    else:
                        self.plot4 = self.scene.mlab.plot3d((0,0),(0,ay),(0,0), color=(0.1,0.1,0.9),tube_radius=0.5, opacity=0.5)
                        self.plot5 = self.scene.mlab.plot3d((ax,ax),(0,ay),(0,0), color=(0.1,0.1,0.9),tube_radius=0.5, opacity=0.5)
                    view(*cameradata)
                elif self.button_state == 1:
#                    path_x, path_y, path_z, path_diameters = build_path_xyz(clustersL, pathL, diameters_fixed, deformation*ax)
                    x1 = [[0., 0.],[0., 0.]]
                    x2 = [[deformation*ax, deformation*ax],[deformation*ax, deformation*ax]]
                    y = [[0., 0.],[ay, ay]]
                    z = [[0., az],[0., az]]
                    cameradata = view()
                    clf()
                    self.plot1 = self.scene.mlab.points3d(deformation*coordx, coordy, coordz, diameters_fixed, color=(0.2, 0.2, 0.2),
                         scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=self.sphere_opacity)
#                    if self.path_state:
#                        self.plot2 = self.scene.mlab.points3d(path_x,path_y,path_z, path_diameters, color=(1.0, 0.2, 0.2),
#                             scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=1.0)    
#                        self.plot3 = self.scene.mlab.plot3d(path_x,path_y,path_z,ones(len(path_x)),color=(0.2,0.9,0.2),tube_radius=0.4, opacity=0.5)
                    if az>0.0:
                        self.plot4 = self.scene.mlab.mesh(x1, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
                        self.plot5 = self.scene.mlab.mesh(x2, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
                    else:
                        self.plot4 = self.scene.mlab.plot3d((0,0),(0,ay),(0,0), color=(0.1,0.1,0.9),tube_radius=0.5, opacity=0.5)
                        self.plot5 = self.scene.mlab.plot3d((deformation*ax,deformation*ax),(0,ay),(0,0), color=(0.1,0.1,0.9),tube_radius=0.5, opacity=0.5)
                    view(*cameradata)
                elif self.button_state == 2:
#                    path_x, path_y, path_z, path_diameters = build_path_xyz(clustersT, pathT, diameters_fixed, ax)
                    x1 = [[0., 0.],[0., 0.]]
                    x2 = [[ax, ax],[ax, ax]]
                    y = [[0., 0.],[deformation*ay, deformation*ay]]
                    z = [[0., max(5.0, az)],[0., max(5.0, az)]]
                    cameradata = view()
                    clf()
                    self.plot1 = self.scene.mlab.points3d(coordx, deformation*coordy, coordz, diameters_fixed, color=(0.2, 0.2, 0.2),
                         scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=self.sphere_opacity)
#                    if self.path_state:
#                        self.plot2 = self.scene.mlab.points3d(path_x,path_y,path_z, path_diameters, color=(1.0, 0.2, 0.2),
#                             scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=1.0)    
#                        self.plot3 = self.scene.mlab.plot3d(path_x,path_y,path_z,ones(len(path_x)),color=(0.2,0.9,0.2),tube_radius=0.4, opacity=0.5)
                    if az>0.0:
                        self.plot4 = self.scene.mlab.mesh(x1, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
                        self.plot5 = self.scene.mlab.mesh(x2, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
                    else:
                        self.plot4 = self.scene.mlab.plot3d((0,0),(0,deformation*ay),(0,0), color=(0.1,0.1,0.9),tube_radius=0.5, opacity=0.5)
                        self.plot5 = self.scene.mlab.plot3d((ax,ax),(0,deformation*ay),(0,0), color=(0.1,0.1,0.9),tube_radius=0.5, opacity=0.5)
                    view(*cameradata)
                gcf().scene.parallel_projection = True
#        view = View(Item('scene', editor=SceneEditor(scene_class=MayaviScene),
#                         height=800, width=1200, show_label=False),
#                    Group(
#                            '_', 'set_unstrained', 'set_strain_longitudinal', 'set_strain_transversal', 'toggle_path',
#                            show_labels=False,
#                         ),
#                    resizable=True,
#                    )

        view = View(HSplit(
                        Item('scene', editor=SceneEditor(scene_class=MayaviScene),
                             height=800, width=1200, show_label=False),
                        Group(
                                '_', 'set_unstrained', 'set_strain_longitudinal', 'set_strain_transversal', 'toggle_path',
                                show_labels=False,
                             ),
                        ),
                        resizable=True,
                    )

    deformation = deformation[0]
    my_model = MyModel()
    my_model.configure_traits()

def MCS_call_model_path(deformation,ax,ay,az,coordx,coordy,coordz,diameters_fixed,clusters0,path0,clustersL,pathL,clustersT,pathT):

    class MyModel(HasTraits):
        axM = ax
        button_state = Int
        set_unstrained = Button()
        set_strain_longitudinal = Button()
        set_strain_transversal = Button()
        path_state = Bool
        toggle_path = Button()
        
        def _set_unstrained_fired(self):
            self.button_state = 0
        def _set_strain_longitudinal_fired(self):
            self.button_state = 1
        def _set_strain_transversal_fired(self):
            self.button_state = 2
        def _toggle_path_fired(self):
            self.path_state = not self.path_state
    
        scene = Instance(MlabSceneModel, (), {'background' : (1,1,1)})
        
        #fig=figure(figure=self.scene.scene,bgcolor=(1,1,1))
    
        plot1 = Instance(PipelineBase)
        plot2 = Instance(PipelineBase)
        plot3 = Instance(PipelineBase)
        plot4 = Instance(PipelineBase)
        plot5 = Instance(PipelineBase)
        
        sphere_resolution = 13  # Auflösung der 3D-Kugeln: 6 für gute Performance, 12 für runde Darstellung
        sphere_opacity = 1.0
        
        # When the scene is activated, or when the parameters are changed, we
        # update the plot.
        @on_trait_change('button_state,scene.activated,path_state')
        def update_plot(self):
            if self.plot1 is None:
                #self.scene.mlab.figure(1, bgcolor=(1, 1, 1), fgcolor=(0, 0, 0), size=(1200, 800))
                #clf()
                self.path_state = True
                clf()
                if az>0.0:
#                    self.plot1 = self.scene.mlab.plot3d((0,ax,ax,0,0),(0,0,ay,ay,0),(0,0,0,0,0),
#                       color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
#                    self.plot2 = self.scene.mlab.plot3d((0,ax,ax,0,0),(0,0,ay,ay,0),(az,az,az,az,az),
#                           color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
#                    self.plot3 = self.scene.mlab.plot3d((0,0),(0,0),(0,az),
#                           color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
#                    self.plot4 = self.scene.mlab.plot3d((ax,ax),(0,0),(0,az),
#                           color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
#                    self.plot5 = self.scene.mlab.plot3d((ax,ax),(ay,ay),(0,az),
#                           color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
#                    self.plot6 = self.scene.mlab.plot3d((0,0),(ay,ay),(0,az),
#                           color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
                             
                    self.plot1 = self.scene.mlab.points3d(coordx, coordy, coordz, diameters_fixed, color=(0.2, 0.2, 0.2),
                             scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=self.sphere_opacity)
                else:
                    # stattdessen Säulen-Darstellung raussuchen?
                    self.plot1 = self.scene.mlab.points3d(coordx, coordy, coordz, diameters_fixed, color=(0.2, 0.2, 0.2),
                             scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=self.sphere_opacity)
                 
                path_x, path_y, path_z, path_diameters = build_path_xyz(clusters0, path0, diameters_fixed, ax)
                self.plot2 = self.scene.mlab.points3d(path_x,path_y,path_z, path_diameters, color=(1.0, 0.2, 0.2),
                     scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=1.0)    
                self.plot3 = self.scene.mlab.plot3d(path_x,path_y,path_z,ones(len(path_x)),color=(0.2,0.9,0.2),tube_radius=0.4, opacity=1.0)
                
                # Elektrodenflächen separat als Rechtecke darstellen
                if az>0.0:
                    x1 = [[0., 0.],[0., 0.]]
                    x2 = [[ax, ax],[ax, ax]]
                    y = [[0., 0.],[ay, ay]]
                    z = [[0., az],[0., az]]
                    self.plot4 = self.scene.mlab.mesh(x1, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
                    self.plot5 = self.scene.mlab.mesh(x2, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
                else:
                    self.plot4 = self.scene.mlab.plot3d((0,0),(0,ay),(0,0), color=(0.1,0.1,0.9),tube_radius=0.5, opacity=0.5)
                    self.plot5 = self.scene.mlab.plot3d((ax,ax),(0,ay),(0,0), color=(0.1,0.1,0.9),tube_radius=0.5, opacity=0.5)
                gcf().scene.parallel_projection = True
            else:
                if self.button_state == 0:
                    path_x, path_y, path_z, path_diameters = build_path_xyz(clusters0, path0, diameters_fixed, ax)
                    x1 = [[0., 0.],[0., 0.]]
                    x2 = [[ax, ax],[ax, ax]]
                    y = [[0., 0.],[ay, ay]]
                    z = [[0., az],[0., az]]
                    cameradata = view()
                    clf()
                    self.plot1 = self.scene.mlab.points3d(coordx, coordy, coordz, diameters_fixed, color=(0.2, 0.2, 0.2),
                         scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=self.sphere_opacity)
                    if self.path_state:
                        self.plot2 = self.scene.mlab.points3d(path_x,path_y,path_z, path_diameters, color=(1.0, 0.2, 0.2),
                             scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=1.0)    
                        self.plot3 = self.scene.mlab.plot3d(path_x,path_y,path_z,ones(len(path_x)),color=(0.2,0.9,0.2),tube_radius=0.4, opacity=1.0)
                    if az>0.0:
                        self.plot4 = self.scene.mlab.mesh(x1, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
                        self.plot5 = self.scene.mlab.mesh(x2, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
                    else:
                        self.plot4 = self.scene.mlab.plot3d((0,0),(0,ay),(0,0), color=(0.1,0.1,0.9),tube_radius=0.5, opacity=0.5)
                        self.plot5 = self.scene.mlab.plot3d((ax,ax),(0,ay),(0,0), color=(0.1,0.1,0.9),tube_radius=0.5, opacity=0.5)
                    view(*cameradata)
                elif self.button_state == 1:
                    path_x, path_y, path_z, path_diameters = build_path_xyz(clustersL, pathL, diameters_fixed, deformation*ax)
                    x1 = [[0., 0.],[0., 0.]]
                    x2 = [[deformation*ax, deformation*ax],[deformation*ax, deformation*ax]]
                    y = [[0., 0.],[ay, ay]]
                    z = [[0., az],[0., az]]
                    cameradata = view()
                    clf()
                    self.plot1 = self.scene.mlab.points3d(deformation*coordx, coordy, coordz, diameters_fixed, color=(0.2, 0.2, 0.2),
                         scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=self.sphere_opacity)
                    if self.path_state:
                        self.plot2 = self.scene.mlab.points3d(path_x,path_y,path_z, path_diameters, color=(1.0, 0.2, 0.2),
                             scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=1.0)    
                        self.plot3 = self.scene.mlab.plot3d(path_x,path_y,path_z,ones(len(path_x)),color=(0.2,0.9,0.2),tube_radius=0.4, opacity=0.5)
                    if az>0.0:
                        self.plot4 = self.scene.mlab.mesh(x1, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
                        self.plot5 = self.scene.mlab.mesh(x2, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
                    else:
                        self.plot4 = self.scene.mlab.plot3d((0,0),(0,ay),(0,0), color=(0.1,0.1,0.9),tube_radius=0.5, opacity=0.5)
                        self.plot5 = self.scene.mlab.plot3d((deformation*ax,deformation*ax),(0,ay),(0,0), color=(0.1,0.1,0.9),tube_radius=0.5, opacity=0.5)
                    view(*cameradata)
                elif self.button_state == 2:
                    path_x, path_y, path_z, path_diameters = build_path_xyz(clustersT, pathT, diameters_fixed, ax)
                    x1 = [[0., 0.],[0., 0.]]
                    x2 = [[ax, ax],[ax, ax]]
                    y = [[0., 0.],[deformation*ay, deformation*ay]]
                    z = [[0., max(5.0, az)],[0., max(5.0, az)]]
                    cameradata = view()
                    clf()
                    self.plot1 = self.scene.mlab.points3d(coordx, deformation*coordy, coordz, diameters_fixed, color=(0.2, 0.2, 0.2),
                         scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=self.sphere_opacity)
                    if self.path_state:
                        self.plot2 = self.scene.mlab.points3d(path_x,path_y,path_z, path_diameters, color=(1.0, 0.2, 0.2),
                             scale_mode='scalar', scale_factor=1.0, resolution=self.sphere_resolution, opacity=1.0)    
                        self.plot3 = self.scene.mlab.plot3d(path_x,path_y,path_z,ones(len(path_x)),color=(0.2,0.9,0.2),tube_radius=0.4, opacity=0.5)
                    if az>0.0:
                        self.plot4 = self.scene.mlab.mesh(x1, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
                        self.plot5 = self.scene.mlab.mesh(x2, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
                    else:
                        self.plot4 = self.scene.mlab.plot3d((0,0),(0,deformation*ay),(0,0), color=(0.1,0.1,0.9),tube_radius=0.5, opacity=0.5)
                        self.plot5 = self.scene.mlab.plot3d((ax,ax),(0,deformation*ay),(0,0), color=(0.1,0.1,0.9),tube_radius=0.5, opacity=0.5)
                    view(*cameradata)
                gcf().scene.parallel_projection = True
#        view = View(Item('scene', editor=SceneEditor(scene_class=MayaviScene),
#                         height=800, width=1200, show_label=False),
#                    Group(
#                            '_', 'set_unstrained', 'set_strain_longitudinal', 'set_strain_transversal', 'toggle_path',
#                            show_labels=False,
#                         ),
#                    resizable=True,
#                    )

        view = View(HSplit(
                        Item('scene', editor=SceneEditor(scene_class=MayaviScene),
                             height=800, width=1200, show_label=False),
                        Group(
                                '_', 'set_unstrained', 'set_strain_longitudinal', 'set_strain_transversal', 'toggle_path',
                                show_labels=False,
                             ),
                        ),
                        resizable=True,
                    )

    my_model = MyModel()
    my_model.configure_traits()

def MCS_plot(number,ax,ay,az,coordx,coordy,coordz,diameters,clusters,path):
    figure(number, bgcolor=(1, 1, 1), fgcolor=(0, 0, 0), size=(1200, 800))
    clf()
    
    if az>0.0:
        plot3d((0,ax,ax,0,0),(0,0,ay,ay,0),(0,0,0,0,0),
           color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
        plot3d((0,ax,ax,0,0),(0,0,ay,ay,0),(az,az,az,az,az),
               color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
        plot3d((0,0),(0,0),(0,az),
               color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
        plot3d((ax,ax),(0,0),(0,az),
               color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
        plot3d((ax,ax),(ay,ay),(0,az),
               color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
        plot3d((0,0),(ay,ay),(0,az),
               color=(0.8,0.8,0.8),tube_radius=0.4, opacity=0.7)
                 
        points3d(coordx, coordy, coordz, diameters, color=(0.2, 0.2, 0.2),
                 scale_mode='scalar', scale_factor=1.0, resolution=10, opacity=1.0)
    else:
        #säulen
        points3d(coordx, coordy, coordz, diameters, color=(0.2, 0.2, 0.2),
                 scale_mode='scalar', scale_factor=1.0, resolution=10, opacity=1.0)
             
    # Pfad darstellen, dabei Elektrodenflächen als ersten und letzten Punkt separat hinzufügen
    path_x = 0.0
    path_y = clusters[path[1],1]
    path_z = clusters[path[1],2]
    for i in range(1,size(path)-1):
        path_x = hstack((path_x,clusters[path[i],0]))
        path_y = hstack((path_y,clusters[path[i],1]))
        path_z = hstack((path_z,clusters[path[i],2]))
    path_x = hstack((path_x,ax))
    path_y = hstack((path_y,clusters[path[-2],1]))
    path_z = hstack((path_z,clusters[path[-2],2]))
    
    path_diameters = hstack((0.,diameters[path[1:-1]],0.))
    points3d(path_x,path_y,path_z, path_diameters, color=(1.0, 0.2, 0.2),
         scale_mode='scalar', scale_factor=1.0, resolution=10, opacity=0.8)    
    plot3d(path_x,path_y,path_z,color=(0.2,0.9,0.2),tube_radius=0.4, opacity=0.5)
    
    # Elektrodenflächen separat als Rechtecke darstellen
    x1 = [[0., 0.],[0., 0.]]
    x2 = [[ax, ax],[ax, ax]]
    y = [[0., 0.],[ay, ay]]
    z = [[0., max(5.0, az)],[0., max(5.0, az)]]
    mesh(x1, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
    mesh(x2, y, z, color=(0.3, 0.3, 0.3), opacity = 0.5)
        
    #points3d(coordx*1.05, coordy, coordz, diameters, color=(0.6, 0.6, 0.6),
    #         scale_mode='scalar', scale_factor=1.0, resolution=6)
    
    #plot3d(coordx[0:4],coordy[0:4],coordz[0:4],color=(0.8,0.2,0.2),tube_radius=1)
    
    view(0,0)
    show()
